<?php

class ModelExtensionModuleMessaggio extends Model
{
    /**
     * @return false|mixed|void
     */
    public function getTemplates()
    {
        $this->load->model('setting/store');

        $templates = [];
        $categories = $this->getCategories();
        if (!empty($categories))
            foreach ($categories as $category)
                $templates[$category['id']] = [];

        $templates_query = $this->db->query("SELECT * FROM " . DB_PREFIX . "messaggio_templates WHERE 1");

        if (!empty($templates_query->rows)) {
            foreach ($templates_query->rows as $row) {
                if ($row['store_id']) {
                    $store = $this->model_setting_store->getStore($row['store_id']);
                    $store = $store['name'];
                } else {
                    $store = $this->language->get('text_default');
                }

                $row['store'] = $store;

                $templates[$row['group']][] = $row;
            }
        }
        return $templates;
    }

    public function getCategories()
    {
        return [
            ['id' => 'order', 'name' => $this->language->get('cat_order'),],
            ['id' => 'payment', 'name' => $this->language->get('cat_payment'),],
            ['id' => 'delivery', 'name' => $this->language->get('cat_delivery'),],
            ['id' => 'admin', 'name' => $this->language->get('cat_admin'),],
            ['id' => 'miscellaneous', 'name' => $this->language->get('cat_miscellaneous'),],
        ];
    }

    public function saveTemplates($id, $data)
    {
        if (empty($id)) {
            if (!empty($data['number']) && is_array($data['number']))
                foreach ($data['number'] as $lang => $number)
                    if (!empty($number) && !empty($data['message'][$lang]))
                        $this->db->query("INSERT INTO " . DB_PREFIX . "messaggio_templates SET `store_id` = '" . (int)$data['store_id'] . "', `group` = '" . $this->db->escape($data['group']) . "', `action` = '" . $this->db->escape($data['action']) . "', `number` = '" . $this->db->escape($number) . "', `message` = '" . $this->db->escape($data['message'][$lang]) . "', `active` = true, `language` = '" . (int)$this->db->escape($lang) . "'");
        } else {
            if (!empty($data['number']) && !empty($data['message']))
                $this->db->query("UPDATE " . DB_PREFIX . "messaggio_templates SET `store_id` = '" . (int)$data['store_id'] . "', `group` = '" . $this->db->escape($data['group']) . "', `action` = '" . $this->db->escape($data['action']) . "', `number` = '" . $this->db->escape($data['number']) . "', `message` = '" . $this->db->escape($data['message']) . "' WHERE `id` = '" . $this->db->escape($id) . "';");
        }
    }

    public function activeTemplates($id, $data)
    {
        if (!empty($id)) {
            $this->db->query("UPDATE " . DB_PREFIX . "messaggio_templates SET `active` = " . ($data['active'] == 'true' ? 'true' : 'false') . " WHERE `id` = '" . $this->db->escape($id) . "';");
        }
    }

    public function deleteTemplates($id)
    {
        if (!empty($id)) {
            $this->db->query("DELETE FROM " . DB_PREFIX . "messaggio_templates  WHERE `id` = '" . $this->db->escape($id) . "';");
        }
    }

    public function getHistory($limit, $offset)
    {
//        $dt_start = new DateTime(!empty($this->request->get['dt-start']) ? $this->request->get['dt-start'] : date("d-m-Y", strtotime("-6 day")));
//        $dt_end = new DateTime(!empty($this->request->get['dt-end']) ? $this->request->get['dt-end'] : date("d-m-Y"));
//        $messaggio_requests = $this->db->query("SELECT * FROM " . DB_PREFIX . "messaggio_requests WHERE `created_at` BETWEEN '" . $dt_start->format('Y-m-d 00:00:00') . "' AND '" . $dt_end->format('Y-m-d 23:59:59') . "' ORDER BY `id` DESC LIMIT " . $limit . " OFFSET " . $offset);
        $messaggio_requests = $this->db->query("SELECT * FROM " . DB_PREFIX . "messaggio_requests WHERE 1 ORDER BY `id` DESC LIMIT " . $limit . " OFFSET " . $offset);
        $messaggio_requests = $messaggio_requests->rows;
        if ($messaggio_requests) {

            $ids = array_diff(array_unique(array_column($messaggio_requests, 'message_id')), ['']);
            $messaggio_response = $ids ? $this->db->query("SELECT * FROM " . DB_PREFIX . "messaggio_responses WHERE `message_id` IN ('" . implode('\', \'', $ids) . "')") : [];

            $messaggio_response_arr = [];
            if (!empty($messaggio_response->rows))
                foreach ($messaggio_response->rows as $response) {
                    $response['error'] = $this->getHookMessage($response['status']);
                    $messaggio_response_arr[$response['message_id']]['array'][] = $response;
                    if ($response['status'] == 70)
                        $messaggio_response_arr[$response->message_id]['delivered'] = true;
                }

            foreach ($messaggio_requests as $key => $request)
                if (!empty($request['message_id']))
                    $messaggio_requests[$key]['response'] = !empty($messaggio_response_arr[$request['message_id']]) ? $messaggio_response_arr[$request['message_id']] : [];

        }
        return $messaggio_requests;
    }

    public function getHookMessage($status)
    {
        $this->load->language('extension/module/messaggio');

        return $this->language->get('hook_status_' . $status);
    }

    public function getHistoryTotal()
    {
        $messaggio_requests = $this->db->query("SELECT count(*) as count FROM " . DB_PREFIX . "messaggio_requests WHERE 1");
        $messaggio_requests = $messaggio_requests->rows;
        return !empty($messaggio_requests[0]['count']) ? $messaggio_requests[0]['count'] : 0;
    }

    public function setHistory($http_code, $errors, $phone, $text, $sending_method, $store_id)
    {
        if (!empty($errors)) {
            foreach ($errors as $error) {
                $this->db->query("INSERT INTO " . DB_PREFIX . "messaggio_requests SET `channel` = '" . (!empty($sending_method) ? $sending_method : 'sms') . "', `phone` = '" . $this->db->escape(!empty($error['phone']) ? $error['phone'] : $phone) . "', `message` = '" . $this->db->escape($text) . "', `message_id` = '" . $this->db->escape(!empty($error['message_id']) ? $error['message_id'] : '') . "', `status` = '" . $this->db->escape($http_code) . "', `error` = '" . $this->db->escape($error['text']) . "', `type` = '" . $this->db->escape($error['type']) . "', `created_at` = '" . date('Y-m-d H:i:s') . "', `updated_at` = '" . date('Y-m-d H:i:s') . "', `store_id` = '" . (int)$this->db->escape($store_id) . "'");
            }
        } else {
            $this->db->query("INSERT INTO " . DB_PREFIX . "messaggio_requests SET `channel` = '" . (!empty($sending_method) ? $sending_method : 'sms') . "', `phone` = '" . $this->db->escape($phone) . "', `message` = '" . $this->db->escape($text) . "', `message_id` = '" . $this->db->escape('') . "', `status` = '" . $this->db->escape($http_code) . "', `error` = '" . $this->db->escape('') . "', `type` = '" . $this->db->escape('') . "', `created_at` = '" . date('Y-m-d H:i:s') . "', `updated_at` = '" . date('Y-m-d H:i:s') . "', `store_id` = '" . (int)$this->db->escape($store_id) . "'");
        }
    }

    public function send($phone, $text, $store_id)
    {
        if (!empty($text)) {
            require_once DIR_SYSTEM . '/library/messaggio/MessaggioSend.php';
            $messaggioSend = new MessaggioSend();
            $messaggioSend->setOptionsCallback($this->url->link('extension/module/messaggio/callback', '', true));
            $messaggioSend->addPhone($phone);

            $messaggio_login = $this->config->get('messaggio_settings_messaggio_login');
            $from = $this->config->get('messaggio_settings_senderid');
            $sending_method = $this->config->get('messaggio_settings_channel');

            switch ($sending_method) {
                case 'viber':
                    $messaggioSend->setFromViber($from);
                    $messaggioSend->addViberMessage($text);
                    break;
                case 'whatsapp':
                    $messaggioSend->setFromWhatsapp($from);
                    $messaggioSend->addWhatsappMessage($text);
                    break;
                case 'vk':
                    $messaggioSend->setFromVk($from);
                    $messaggioSend->addVkMessage($text);
                    break;
                default:
                    $messaggioSend->setFromSms($from);
                    $messaggioSend->addSmsMessage($text);
            }
            $messaggioSend->setMessaggioLogin($messaggio_login);
            list($http_code,) = $messaggioSend->send();
            $errors = $messaggioSend->getErrors();

            $this->setHistory($http_code, $errors, $phone, $text, !empty($settings['sending_method']) ? $settings['sending_method'] : '', $store_id);
        }
    }

    private function shortcodeParseAtts($text)
    {
        $atts = [];
        $pattern = '/([\w-]+)\s*=\s*"([^"]*)"(?:\s|$)|([\w-]+)\s*=\s*\'([^\']*)\'(?:\s|$)|([\w-]+)\s*=\s*([^\s\'"]+)(?:\s|$)|"([^"]*)"(?:\s|$)|\'([^\']*)\'(?:\s|$)|(\S+)(?:\s|$)/';
        $text = preg_replace("/[\x{00a0}\x{200b}]+/u", " ", $text);
        if (preg_match_all($pattern, $text, $match, PREG_SET_ORDER)) {
            foreach ($match as $m) {
                if (!empty($m[1]))
                    $atts[strtolower($m[1])] = stripcslashes($m[2]);
                else if (!empty($m[3]))
                    $atts[strtolower($m[3])] = stripcslashes($m[4]);
                else if (!empty($m[5]))
                    $atts[strtolower($m[5])] = stripcslashes($m[6]);
                else if (isset($m[7]) && strlen($m[7]))
                    $atts[] = stripcslashes($m[7]);
                else if (isset($m[8]) && strlen($m[8]))
                    $atts[] = stripcslashes($m[8]);
                else if (isset($m[9]))
                    $atts[] = stripcslashes($m[9]);
            }

            // Reject any unclosed HTML elements
            foreach ($atts as &$value) {
                if (false !== strpos($value, '<')) {
                    if (1 !== preg_match('/^[^<]*+(?:<[^>]*+>[^<]*+)*+$/', $value)) {
                        $value = '';
                    }
                }
            }
        } else {
            $atts = ltrim($text);
        }
        return $atts;
    }

    public function parseShortcodeOrder($text, $order, $return_order = false)
    {
        $shortcodes = [
            "order_id",
            "firstname",
            "lastname",
            "email",
            "telephone",
            "date_added",
            "payment_method",
            "shipping_method",
            "ip",
            "payment_address",
            "shipping_address",
            "products",
            "order_comment",
            "store_name",
            "store_url",
            "order_amount",
        ];
        foreach ($shortcodes as $shortcode) {
            if (preg_match_all('|\[' . $shortcode . '(.*)\]|isU', $text, $arr)) {
                if (!empty($arr[0])) {
                    foreach ($arr[0] as $key => $value) {
                        $attr = $this->shortcodeParseAtts(!empty($arr[1][$key]) ? $arr[1][$key] : '');
                        $before = !empty($attr['before']) ? $attr['before'] : '';
                        $after = !empty($attr['after']) ? $attr['after'] : '';

                        $replace_text = '';
                        switch ($shortcode) {
                            case "order_id":
                                if ($return_order)
                                    $replace_text = !empty($return_order['order_id']) ? $return_order['order_id'] : '';
                                else
                                    $replace_text = !empty($order['order_id']) ? $order['order_id'] : '';
                                break;
                            case "firstname":
                                if ($return_order)
                                    $replace_text = !empty($return_order['firstname']) ? $return_order['firstname'] : (!empty($order['firstname']) ? $order['firstname'] : (!empty($order['payment_firstname']) ? $order['payment_firstname'] : (!empty($order['shipping_firstname']) ? $order['shipping_firstname'] : '')));
                                else
                                    $replace_text = !empty($order['firstname']) ? $order['firstname'] : (!empty($order['payment_firstname']) ? $order['payment_firstname'] : (!empty($order['shipping_firstname']) ? $order['shipping_firstname'] : ''));
                                break;
                            case "lastname":
                                if ($return_order)
                                    $replace_text = !empty($return_order['lastname']) ? $return_order['lastname'] : (!empty($order['lastname']) ? $order['lastname'] : (!empty($order['payment_lastname']) ? $order['payment_lastname'] : (!empty($order['shipping_lastname']) ? $order['shipping_lastname'] : '')));
                                else
                                    $replace_text = !empty($order['lastname']) ? $order['lastname'] : (!empty($order['payment_lastname']) ? $order['payment_lastname'] : (!empty($order['shipping_lastname']) ? $order['shipping_lastname'] : ''));
                                break;
                            case "email":
                                if ($return_order)
                                    $replace_text = !empty($return_order['email']) ? $return_order['email'] : '';
                                else
                                    $replace_text = !empty($order['email']) ? $order['email'] : '';
                                break;
                            case "telephone":
                                if ($return_order)
                                    $replace_text = !empty($return_order['telephone']) ? $return_order['telephone'] : '';
                                else
                                    $replace_text = !empty($order['telephone']) ? $order['telephone'] : '';
                                break;
                            case "date_added":
                                if ($return_order)
                                    $replace_text = !empty($return_order['date_added']) ? $return_order['date_added'] : '';
                                else
                                    $replace_text = !empty($order['date_added']) ? $order['date_added'] : '';
                                break;
                            case "payment_method":
                                $replace_text = !empty($order['payment_method']) ? $order['payment_method'] : '';
                                break;
                            case "shipping_method":
                                $replace_text = !empty($order['shipping_method']) ? $order['shipping_method'] : '';
                                break;
                            case "ip":
                                $replace_text = !empty($order['forwarded_ip']) ? $order['forwarded_ip'] : '';
                                break;
                            case "payment_address":
                                $replace_text = !empty($order['payment_address_1']) ? $order['payment_address_1'] : '';
                                break;
                            case "shipping_address":
                                $replace_text = !empty($order['shipping_address_1']) ? $order['shipping_address_1'] : '';
                                break;
                            case "products":
                                $replace_text = '';

                                if ($return_order)
                                    $replace_text = !empty($return_order['product']) ? ($return_order['product'] . ' * ' . $return_order['quantity']) : '';
                                else {
                                    $order_product_query = $this->db->query("SELECT * FROM " . DB_PREFIX . "order_product WHERE order_id = '" . (int)(!empty($order['order_id']) ? $order['order_id'] : '') . "'");
                                    foreach ($order_product_query->rows as $product) {
                                        if (end($order_product_query->rows) == $product)
                                            $replace_text .= $product['name'] . ' * ' . $product['quantity'];
                                        else
                                            $replace_text .= $product['name'] . ' * ' . $product['quantity'] . ', ';
                                    }
                                }

                                break;
                            case "order_comment":
                                if ($return_order)
                                    $replace_text = !empty($return_order['comment']) ? $return_order['comment'] : '';
                                else
                                    $replace_text = !empty($order['comment']) ? $order['comment'] : '';
                                break;
                            case "store_name":
                                $replace_text = !empty($order['store_name']) ? $order['store_name'] : '';
                                break;
                            case "store_url":
                                $replace_text = !empty($order['store_url']) ? $order['store_url'] : '';
                                break;
                            case "order_amount":
                                if ($return_order)
                                    $replace_text = '';
                                else
                                    $replace_text = $this->currency->format($order['total'], $order['currency_code']);
                                break;
                        }
                        if (!empty($replace_text))
                            $replace_text = $before . $replace_text . $after;

                        $text = str_replace($value, $replace_text, $text);
                    }
                }
            }
        }
        return $text;
    }

    public function getTemplateByAction($store_id, $action, $language_id)
    {
        if ($store_id === false && $language_id === false) {
            $templates = $this->db->query("SELECT * FROM " . DB_PREFIX . "messaggio_templates WHERE `action` = '" . $this->db->escape($action) . "' AND `active` = true");
        } else
            $templates = $this->db->query("SELECT * FROM " . DB_PREFIX . "messaggio_templates WHERE `store_id` = '" . (int)$store_id . "' AND `language` = '" . (int)$language_id . "' AND `action` = '" . $this->db->escape($action) . "' AND `active` = true");
        return $templates->rows;
    }
}

