<?php
class MessaggioSend
{

    private $url = 'https://msg.messaggio.com/api/v1/send';
    private $messaggioLogin = '';
    private $channels = [];
    private $phone = [];

    private $sms = [];
    private $fromSms = '';

    private $viber = [];
    private $fromViber = '';

    private $whatsapp = [];
    private $fromWhatsapp = '';

    private $vk = [];
    private $fromVk = '';

    private $options = [
        "ttl" => 30,
    ];

    private $errors = [];

    /**
     * MessaggioSend constructor.
     */
    public function __construct()
    {
    }

    public function getData()
    {
        $recipients = [];
        if (!empty($this->phone))
            foreach ($this->phone as $phone)
                $recipients[] = ['phone' => $phone];

        $data = [
            'recipients' => $recipients,
            'channels'   => $this->getChannels(),
            'options'    => $this->getOptions(),
        ];

        if (!empty($this->sms)) {
            $sms = [
                "from" => $this->getFromSms(),
            ];
            foreach ($this->sms as $text)
                $sms['content'][] = ["type" => "text", "text" => $text];

            $data['sms'] = $sms;
        }

        if (!empty($this->viber)) {
            $viber = [
                "from"  => $this->getFromViber(),
                "label" => 'transaction',
            ];
            foreach ($this->viber as $text)
                $viber['content'][] = ["type" => "text", "text" => $text];

            $data['viber'] = $viber;
        }

        if (!empty($this->whatsapp)) {
            $whatsapp = [
                "from" => $this->getFromWhatsapp(),
            ];
            foreach ($this->whatsapp as $text)
                $whatsapp['content'][] = ["type" => "text", "text" => $text];

            $data['whatsapp'] = $whatsapp;
        }

        if (!empty($this->vk)) {
            $vk = [
                "from" => $this->getFromVk(),
            ];
            foreach ($this->vk as $text)
                $vk['content'][] = ["type" => "text", "text" => $text];

            $data['vk'] = $vk;
        }

        return $data;
    }

    public function send()
    {
        $data = $this->getData();
        $ch = curl_init($this->url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data, JSON_UNESCAPED_UNICODE));
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'accept: application/json',
            'Messaggio-Login: ' . $this->getMessaggioLogin(),
        ]);
        curl_setopt($ch, CURLOPT_TIMEOUT, 45);
        $output = curl_exec($ch);
        $info = curl_getinfo($ch);
        curl_close($ch);
        $this->checkErrors($info['http_code'], $output);

        return [$info['http_code'], $output];
    }

    private function checkErrors($http_code, $output)
    {
        $output = json_decode($output, true);
        switch ($http_code) {
            case 0:
                $this->addError(['type' => 'danger', 'text' => 'Your request has been sent, but the result of the operation has not been received']);
                break;
            case 500:
                $this->addError(['type' => 'danger', 'text' => 'Internal error, try again later']);
                break;
            case 200:
                if (!empty($output['messages']))
                    foreach ($output['messages'] as $message) {
                        $text = '';
                        if (isset($message['recipient']['phone']))
                            $text .= $message['recipient']['phone'] . ' - ';
                        if (isset($message['error'])) {
                            $text .= $message['error']['title'] . ' (' . $message['error']['detail'] . ')';
                            $this->addError(['type' => 'danger', 'text' => $text, 'phone' => !empty($message['recipient']['phone']) ? $message['recipient']['phone'] : null]);
                        } else {
                            $text .= 'Request accepted for processing';
                            $this->addError(['type' => 'success', 'text' => $text, 'message_id' => $message['message_id'], 'phone' => !empty($message['recipient']['phone']) ? $message['recipient']['phone'] : null]);
                        }
                    }
                break;
            default:
                $this->addError(['type' => 'warning', 'text' => $output['title'] . ' (' . $output['detail'] . ')']);
                break;

        }
    }

    /**
     * @return array
     */
    public function getErrors()
    {
        return $this->errors;
    }

    /**
     * @param array $error
     */
    public function addError(array $error)
    {
        $this->errors[] = $error;
    }


    /**
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * @param string $url
     */
    public function setUrl($url)
    {
        $this->url = $url;
    }

    /**
     * @return array
     */
    public function getChannels()
    {
        return $this->channels;
    }

    /**
     * @param string $text
     */
    public function addSmsMessage($text)
    {
        $this->channels = array_merge($this->channels, ['sms']);
        if (!empty($text)) {
            $this->sms[] = trim($text);
        }
    }

    /**
     * @param string $text
     */
    public function addViberMessage($text)
    {
        $this->channels = array_merge($this->channels, ['viber']);
        if (!empty($text)) {
            $this->viber[] = trim($text);
        }
    }

    /**
     * @param string $text
     */
    public function addWhatsappMessage($text)
    {
        $this->channels = array_merge($this->channels, ['whatsapp']);
        if (!empty($text)) {
            $this->whatsapp[] = trim($text);
        }
    }

    /**
     * @param string $text
     */
    public function addVkMessage($text)
    {
        $this->channels = array_merge($this->channels, ['vk']);
        if (!empty($text)) {
            $this->vk[] = trim($text);
        }
    }

    /**
     * @return array
     */
    public function getPhones()
    {
        return $this->phone;
    }

    /**
     * @param string $phone
     */
    public function addPhone($phone)
    {
        $this->phone[] = preg_replace("/[^0-9]/", '', $phone);
    }

    /**
     *
     * Количество секунд, по истечению которых сообщение считается недоставленным в канал и начинается отправка сообщения в следующий канал. Минимальное значение этого параметра равно 30. Значение по умолчанию равно 30.
     *
     * @param int $ttl
     */
    public function setOptionsTtl($ttl)
    {
        $this->options['ttl'] = $ttl;
    }

    /**
     * Используется для переопределения адреса отправки статусов доставки сообщения. По умолчанию используется значение из настроек проекта в личном кабинете Messaggio.
     *
     * @param string $callback
     */
    public function setOptionsCallback($callback)
    {
        if (!empty($callback))
            $this->options['dlr_callback_url'] = $callback;
        else unset($this->options['dlr_callback_url']);
    }

    /**
     * Служит для идентификации сообщений пользователем в системе и передается вместе со статусом доставки
     *
     * @param string $external_id
     */
    public function setOptionsExternalId($external_id)
    {
        if (!empty($external_id))
            $this->options['external_id'] = $external_id;
        else unset($this->options['external_id']);
    }

    /**
     * @return int[]
     */
    public function getOptions()
    {
        return $this->options;
    }


    /**
     * @return string
     */
    public function getFromSms()
    {
        return $this->fromSms;
    }

    /**
     *
     * Имя клиентского отправителя в личном кабинете Messaggio
     *
     * @param string $fromSms
     */
    public function setFromSms($fromSms)
    {
        $this->fromSms = $fromSms;
    }

    /**
     * @return string
     */
    public function getFromViber()
    {
        return $this->fromViber;
    }

    /**
     *
     * Имя клиентского отправителя в личном кабинете Messaggio
     *
     * @param string $fromViber
     */
    public function setFromViber($fromViber)
    {
        $this->fromViber = $fromViber;
    }

    /**
     * @return array
     */
    public function getWhatsapp()
    {
        return $this->whatsapp;
    }

    /**
     *
     *  Имя клиентского отправителя в личном кабинете Messaggio
     *
     * @param array $whatsapp
     */
    public function setWhatsapp(array $whatsapp)
    {
        $this->whatsapp = $whatsapp;
    }

    /**
     * @return string
     */
    public function getFromWhatsapp()
    {
        return $this->fromWhatsapp;
    }

    /**
     *
     *  Имя клиентского отправителя в личном кабинете Messaggio
     *
     * @param string $fromWhatsapp
     */
    public function setFromWhatsapp($fromWhatsapp)
    {
        $this->fromWhatsapp = $fromWhatsapp;
    }

    /**
     * @return string
     */
    public function getFromVk()
    {
        return $this->fromVk;
    }

    /**
     *
     * Имя клиентского отправителя в личном кабинете Messaggio
     *
     * @param string $fromVk
     */
    public function setFromVk($fromVk)
    {
        $this->fromVk = $fromVk;
    }

    /**
     * @return string
     */
    public function getMessaggioLogin()
    {
        return $this->messaggioLogin;
    }

    /**
     *
     * Логин проекта в системе Messaggio
     *
     * @param string $messaggioLogin
     */
    public function setMessaggioLogin($messaggioLogin)
    {
        $this->messaggioLogin = $messaggioLogin;
    }
}
